
//====[Ʀ즡Lu CH 1~6 + 3 Switch ]==========================================

//====[TWI(I2C) DҦ MASTER]===========================================

#include <stdlib.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include <compat/twi.h>

#define MAX_TRIES 50

#define I2C_START 0
#define I2C_DATA 1
#define I2C_DATA_ACK 2
#define I2C_STOP 3
#define ACK 1
#define NACK 0


extern volatile unsigned int VAL_ch1,VAL_ch2,VAL_ch3,VAL_ch4,VAL_ch5,VAL_ch6;
extern volatile unsigned int VAL_buttons;

volatile unsigned int VAL_ch1 = 0;
volatile unsigned int VAL_ch2 = 0;
volatile unsigned int VAL_ch3 = 0;
volatile unsigned int VAL_ch4 = 0;
volatile unsigned int VAL_ch5 = 0;
volatile unsigned int VAL_ch6 = 0;

volatile unsigned int VAL_buttons = 0;

//====[START I2C Routine]===========================================================================================

unsigned char i2c_transmit(unsigned char type) {

	switch(type){
		case I2C_START:    											//===[Send Start Condition]==============
    	 	 TWCR = (1 << TWINT) | (1 << TWSTA) | (1 << TWEN);
       		  break;
 	    case I2C_DATA:    											//===[Send Data with No-Acknowledge]=====
			 TWCR = (1 << TWINT) | (1 << TWEN);
     		 break;
     	case I2C_DATA_ACK: 											//===[Send Data with Acknowledge]========
      		 TWCR = (1 << TWEA) | (1 << TWINT) | (1 << TWEN);
      		 break;
  		case I2C_STOP:   											//===[Send Stop Condition]===============
		 	 TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWSTO);
			 return 0;
	}


	//====[Wait for TWINT flag set on Register TWCR]=======================================
	while (!(TWCR & (1 << TWINT)));

	//====[Return TWI Status Register, mask the prescaler bits (TWPS1,TWPS0)]==============
  	return (TWSR & 0xF8);
}

char i2c_start( unsigned char servoid, unsigned char rw_type)
{
	unsigned char n = 0;
	unsigned char twi_status;

	char r_val = -1;

i2c_retry:

	if (n++ >= MAX_TRIES) return r_val;

	//====[Transmit Start Condition]============================
	twi_status=i2c_transmit(I2C_START);

	//====[Check the TWI Status]==================================================================
	if (twi_status == TW_MT_ARB_LOST) goto i2c_retry;

	if ((twi_status != TW_START) && (twi_status != TW_REP_START)) goto i2c_quit;

	//====[Send slave address (SLA_W or SLA_R)]=================
  	servoid = servoid << 1;
  	TWDR = (servoid & 0xFE) | rw_type;

  	//====[Transmit I2C Data]===================================
  	twi_status=i2c_transmit(I2C_DATA);

  	//====[Check the TWSR status]=================================================================
  	if (rw_type){

  		if ((twi_status == TW_MR_SLA_NACK) || (twi_status == TW_MR_ARB_LOST)) goto i2c_retry;
  		if (twi_status != TW_MR_SLA_ACK) goto i2c_quit;

  	}else{

  		if ((twi_status == TW_MT_SLA_NACK) || (twi_status == TW_MT_ARB_LOST)) goto i2c_retry;
  		if (twi_status != TW_MT_SLA_ACK) goto i2c_quit;

  	}	

  	r_val=0;

i2c_quit:

  	return r_val;
}


void i2c_stop(void)
{
  	unsigned char twi_status3;

  	//====[Transmit I2C Data]=====================
  	twi_status3 =i2c_transmit(I2C_STOP);
}

unsigned int TWI_READ(unsigned char servoid, unsigned char reg_id)
{

	unsigned char twi_status,REG_VH,REG_VL;
	unsigned int rv;
  	char r_val;

	r_val = 0;
	twi_status = 0;

	REG_VH = 0;
	REG_VL = 0;
	rv = 0;

	//====[}l]======================================================
	r_val = i2c_start(servoid,TW_WRITE);

	//====[ǭ]==========================================
  	if (r_val == 0){

		//====[ǰe REG_ID ]==============================
		TWDR = reg_id;
  		twi_status=i2c_transmit(I2C_DATA);
	}  

   	//====[]=======
   	i2c_stop();


	//====[}l]======================================================
    r_val = i2c_start(servoid,TW_READ);

	//====[ǭ]=======================================================================
  	if (r_val == 0){
		
  		twi_status=i2c_transmit(I2C_DATA_ACK);
  		//  = 0x50   data received, ACK returned

   		REG_VH = TWDR;

		//   twi_status=i2c_transmit(I2C_DATA);
  		twi_status=i2c_transmit(I2C_DATA);
  		//  = 0x58    data received, NACK returned

  		REG_VL = TWDR;

	}

   	//====[]=======
	i2c_stop();

   	//====[]==================
	rv = (REG_VH << 8);
	rv = rv | REG_VL;

   	//====[^ǭ]==================
	return rv;
}

//====[TWI/I2C ǭ]================================================================================

void TWI_WRITE(unsigned char servoid, unsigned char reg_id, unsigned int reg_value)
{

	unsigned char twi_status;
	char r_val = 0;

	//====[}l]======================================================
	r_val = i2c_start(servoid,TW_WRITE);

	//====[ǭ]==========================================
  	if (r_val == 0){

		//====[ǰe REG_ID ]==============================
		TWDR = reg_id;
  		twi_status=i2c_transmit(I2C_DATA);

		//====[ǰ8줸]==============================
		TWDR = (reg_value >> 8);
  		twi_status=i2c_transmit(I2C_DATA);

		//====[ǧC8줸]==============================
		TWDR = reg_value;
	  	twi_status=i2c_transmit(I2C_DATA);

	}  

   	//====[]=======
   	i2c_stop();
   
}



//=====[ICP TWI MASTER]===========================================

int main(void)
{

	cli(); 

	MCUCR  = 0x00;
	MCUCSR = 0x80;
	GICR   = 0x00;


	DDRB  = 0xFE;
	PORTB = 0x00;

	PORTC = 0x00;
	DDRC  = 0x00;


	DDRD  = 0xFF;
	PORTD = 0x00;


	//====[TWI init]==========================================================================================
  	TWSR = 0x00;        //====[ Select Prescaler of 1 	
  	TWBR = 0x30;        //====[11059200 / (16 + 2 * 48 * 1) = 98.743 kHz 48 Decimal = 0x30  => 2ms ȶg
/*
	//====[ADC init]===================================================================================

	//====[CLCK / 128 ̤pWv ]=======================================
	ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1)  | (1<<ADPS0);
	
	// Set ADMUX Register on ATMega8
	ADMUX = 0x40;       // Use Left Justified, Select Channel 0
*/
	//====[ɰ_(iٲ)]========
	_delay_ms(1000);

	sei();

	while(1){
/*
		//====[ADC0 (׻)]==================================================================================
		ADMUX = 0x40;  

		//====[ADC }lŪq]====================
		ADCSRA |= (1<<ADSC);

		while (ADCSRA & (1<<ADSC));		//====[ AD ഫ]==========
		
		VAL_ch1 = ADCL;
		VAL_ch1 = VAL_ch1 | ((ADCH & 0x03) << 8);

		ADCSRA |= _BV(ADIF);					//====[MX]=======================


		TWI_WRITE(1, 1, VAL_ch1);					//====[REG_ID: 1 (n1)]=======
*/
		//====[zL TWI(I2C)ǭ]============================================================================

		//====[RADIO ŪLuҲձ]===================================================

		VAL_ch1 = TWI_READ(0x11, 1);
		VAL_ch2 = TWI_READ(0x11, 2);
		VAL_ch3 = TWI_READ(0x11, 3);
		VAL_ch4 = TWI_READ(0x11, 4);
		VAL_ch5 = TWI_READ(0x11, 5);
		VAL_ch6 = TWI_READ(0x11, 6);

		VAL_buttons = TWI_READ(0x11, 7); 

		//====[DISPLAY 16x02]================================================================
										
		TWI_WRITE(1, 1, VAL_ch1);					//====[REG_ID: 1 (n1)]=======
		TWI_WRITE(1, 2, VAL_ch2);					//====[REG_ID: 2 (n2)]=======
		TWI_WRITE(1, 3, VAL_ch3);					//====[REG_ID: 3 (n3)]=======
		TWI_WRITE(1, 4, VAL_ch4);					//====[REG_ID: 4 (n4)]=======
		TWI_WRITE(1, 5, VAL_ch5);					//====[REG_ID: 5 (n5)]=======
		TWI_WRITE(1, 6, VAL_ch6);					//====[REG_ID: 6 (n6)]=======

		TWI_WRITE(1, 7, VAL_buttons);				//====[REG_ID: 7 (s)]======= 

		//====[PC MONITOR]=================================================================

//		TWI_WRITE(2, 9, VAL_angle);					//====[REG_ID: 9 (׭)]=======
//		TWI_WRITE(2,10, VAL_gyro);					//====[REG_ID:10 (t)]=======	

		TWI_WRITE(2, 1, VAL_ch1);					//====[REG_ID: 1 (n1)]=======
		_delay_us(50);
		TWI_WRITE(2, 2, VAL_ch2);					//====[REG_ID: 2 (n2)]=======
		_delay_us(50);
		TWI_WRITE(2, 3, VAL_ch3);					//====[REG_ID: 3 (n3)]=======
		_delay_us(50);
		TWI_WRITE(2, 4, VAL_ch4);					//====[REG_ID: 4 (n4)]=======
		_delay_us(50);
		TWI_WRITE(2, 5, VAL_ch5);					//====[REG_ID: 5 (n5)]=======
		_delay_us(50);
		TWI_WRITE(2, 6, VAL_ch6);					//====[REG_ID: 6 (n6)]=======
		_delay_us(50);
		TWI_WRITE(2, 20, VAL_buttons);				//====[REG_ID: 20 (s)]=======
						
		//====[]=========== 
		_delay_ms(25);

	} 
}


